<?php

// Controlador que alberga la lógica de gestión de las colecciones.

namespace App\Http\Controllers;

use App\Exposicion;
use Illuminate\Http\Request;
use App\Http\Requests\ActualizarColeccionRequest;

class ExposicionesController extends Controller
{
  // El uso de este controlador requerirá de la superación del middleware auth, es decir,
  // que el usuario haya iniciado sesión en la aplicación
  function __construct()
  {
    $this->middleware('auth');
  }
  
  /**
  * Listado de colecciones.
  */
  public function index()
  {
    // Obtenemos el listado de colecciones del usuario autenticado y se lo pasamos a la vista colecciones.index
    $exposiciones = auth()->user()->exposiciones;
    return view('colecciones.index', compact('exposiciones'));
  }
  
  /**
  * Creación de colección --> No implementado en versión release
  */
  public function create()
  {
    //
  }
  
  /**
  * Guardar colección en DB --> No implementado en versión release
  */
  public function store(Request $request)
  {
    //
  }
  
  /**
  * Ver colección especificada por $id.
  */
  public function show($id)
  {
    // Localizamos la colección o lanzamos error.
    $exposicion = Exposicion::findOrFail($id);
    // ¿Tiene el usuario autenticado permiso para ver la colección?
    // La lógica de autorización está en app/Policies/ExposicionesPolicy.php
    $this->authorize('show', $exposicion);
    // Si tiene autorización para ver la colección, le pasamos los datos de $exposicion
    return view('colecciones.show',compact('exposicion'));
  }
  
  /**
  * Formulario de edición de la colección.
  */
  public function edit($id)
  {
    // Localizamos la colección o lanzamos error.
    $exposicion = Exposicion::findOrFail($id);
    // ¿Tiene el usuario autenticado permiso para ver el formulario de edición de la colección?
    // La lógica de autorización está en app/Policies/ExposicionesPolicy.php
    $this->authorize('edit', $exposicion);
    // Si tiene autorización para ver el formulario de edición, le pasamos los datos de $exposicion
    return view('colecciones.edit',compact('exposicion'));
  }
  
  /**
  * Actualizar la colección.
  * Para validar los datos de actualización, se ha implementado la lógica de validación en una clase Request,
  * ubicada en app/Http/Requests/ActualizarColeccionRequest.php
  */
  public function update(ActualizarColeccionRequest $request, $id)
  {
    // Localizamos la colección o lanzamos error.
    $exposicion = Exposicion::findOrFail($id);
    // ¿Tiene el usuario autenticado permiso para actualizar la colección?
    // La lógica de autorización está en app/Policies/ExposicionesPolicy.php
    $this->authorize('update', $exposicion);
    // Si tiene autorización, actualizamos la colección.
    // Para actualizar, usamos la asignación masiva (no hacemos asignación individual de cada campo)
    $exposicion->update($request->all());
    
    // Redirigimos al listado de colecciones con feedback
    return redirect()->route('colecciones.index')->with(['tipo' =>'success','info' =>'Colección actualizada correctamente']);
  }
  
  /**
  * Eliminación de colección --> No implementado en versión release.
  */
  public function destroy(Exposicion $exposicion)
  {
    //
  }
  
  /**
  * Publicación de colección --> la ficha pública será accesible en el rango de fechas especificado por el user en DB.
  */
  public function publicar_coleccion($id)
  {
    // Localizamos la colección o lanzamos error.
    $exposicion = Exposicion::findOrFail($id);
    // ¿Tiene el usuario autenticado permiso para publicar la colección?
    // La lógica de autorización está en app/Policies/ExposicionesPolicy.php
    $this->authorize('publicar_coleccion', $exposicion);
    // Si la colección es pública, la ocultamos. Si está oculta, la publicamos
    // Esta implementación responde a la necesidad de que el administrador puede activar/desactivar las colecciones
    // El resto de usuarios sólo verá el botón 'publicar' (activar) si las colecciones están desactivadas
    $toggle = ($exposicion->esPublica) ? false : true;
    $exposicion->esPublica = $toggle;
    // Persistencia en DB.
    $exposicion->save();
    // Redirigimos al listado de colecciones con feedback
    return redirect()
    ->route('colecciones.index')
    ->with(['tipo' =>'success','info' =>'Colección publicada correctamente']);
  }
}
