<?php

// Controlador que alberga la lógica de gestión de las obras.

namespace App\Http\Controllers;

use DB;
use App\Obra;
use App\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use App\Http\Requests\ValidarNuevaObra;


class ObrasController extends Controller
{
  // El uso de este controlador requerirá de la superación del middleware auth, es decir,
  // que el usuario haya iniciado sesión en la aplicación
  function __construct()
  {
    $this->middleware('auth');
  }
  
  /**
  * Listado de obras del usuario.
  */
  public function index()
  {
    // Obtenemos el listado de obras del usuario autenticado y se lo pasamos a la vista obras.index
    $obras = auth()->user()->obras;
    return view('obras.index', compact('obras'));
  }
  
  /**
  * Creación de obra --> Formulario de alta de nueva obra.
  */
  public function create()
  {
    // Obtenemos el listado de colecciones del usuario autenticado y se lo pasamos a la vista obras.create
    $exposiciones = auth()->user()->exposiciones->pluck('Titulo','id');
    return view('obras.create', compact('exposiciones'));
  }
  
  /**
  * Guardar obra en DB
  * Para validar los datos recibidos, se ha implementado la lógica de validación en una clase Request,
  * ubicada en app/Http/Requests/ValidarNuevaObra.php
  */
  public function store(ValidarNuevaObra $request)
  {
    // Creamos una nueva obra y le asignamos los datos recibidos
    $obra = Obra::create([
      'Titulo' => $request->get('Titulo'),
      'Autor' => $request->get('Autor'),
      'URL' => $request->get('URL'),
      'Fecha' => $request->get('Fecha'),
      'CuerpoWeb' => $request->get('CuerpoWeb'),
      'Tecnica' => $request->get('Tecnica'),
      'Medidas' => $request->get('Medidas'),
      'Planta' => $request->get('Planta'),
      'Ubicacion' => $request->get('Ubicacion'),
      'Foto' => $request->get('Thumb'),
      'Thumb' => $request->get('Thumb'),
      'user_id' => auth()->user()->id,
      'exposicion_id' => $request->get('exposicion_id'),
      'Descripcion' => $request->get('Descripcion'),
      'esPublica' => false, // por defecto, la obra no se publica
      'obra_token' => str_random(12), // identificador único aleatorio de la obra
      ]
    );
    
    // Redirigimos al listado de obras con feedback
    return redirect()->route('obras.index')->with(['tipo' =>'success','info' =>'Nueva obra insertada']);
  }
  
  /**
  * Ver obra especificada por $id.
  */
  public function show($id)
  {
    // Localizamos la obra o lanzamos error.
    $obra = Obra::findOrFail($id);
    // ¿Tiene el usuario autenticado permiso para ver la obra?
    // La lógica de autorización está en app/Policies/ObraPolicy.php
    $this->authorize('show', $obra);
    // Si tiene autorización, redirigimos a la vista obras.ficha-publica con los datos de la $obra
    return view('obras.ficha-publica',compact('obra'));
  }
  
  /**
  * Formulario de edición de la obra.
  */
  public function edit($id)
  {
    // Localizamos la colección o lanzamos error.
    $obra = Obra::findOrFail($id);
    // ¿Tiene el usuario autenticado permiso para ver el formulario de edición de la obra?
    // La lógica de autorización está en app/Policies/ObraPolicy.php
    $this->authorize('edit', $obra);
    // Si tiene autorización, recuperamos su listado de colecciones y redirigimos a la vista obras.edit
    $exposiciones = auth()->user()->exposiciones->pluck('Titulo','id');
    return view('obras.edit',compact('obra','exposiciones'));
  }
  
  /**
  * Actualizar la obra.
  */
  public function update(Request $request, $id)
  {
    // Localizamos la obra o lanzamos error.
    $obra = Obra::findOrFail($id);
    // ¿Tiene el usuario autenticado permiso para actualizar la obra?
    // La lógica de autorización está en app/Policies/ObraPolicy.php
    $this->authorize('update', $obra);
    // Si tiene autorización, actualizamos la obra.
    // Para actualizar, usamos la asignación masiva (no hacemos asignación individual de cada campo).
    // Luego, redirigmos al listado de obras, con feedback
    $obra->update($request->all());
    return redirect()->route('obras.index')->with(['tipo' =>'success','info' =>'Obra actualizada correctamente']);
  }
  
  /**
  * Eliminar la obra.
  */
  public function destroy($id)
  {
    // Localizamos la obra o lanzamos error.
    $obraABorrar = Obra::findOrFail($id);
    // ¿Tiene el usuario autenticado permiso para eliminar la obra?
    // La lógica de autorización está en app/Policies/ObraPolicy.php
    $this->authorize('destroy', $obraABorrar);
    // Tiene permiso --> localizamos la imagen de la obra si la tiene y la eliminamos de almacenamiento
    $fotoURL = str_replace('storage','public',$obraABorrar->Foto);
    Storage::delete($fotoURL);
    // Elimimamos la obra en DB y redirigimos con feedback para el usuario.
    $obraABorrar->delete();
    return redirect()->route('obras.index')->with(['tipo' =>'success','info' =>'Obra eliminada correctamente']);
  }
  
  /**
  * Mostramos la vista de generación en lote de todas las etiquetas del usuario.
  */
  public function generar_mis_etiquetas($id)
  {
    // Localizamos el usuario o lanzamos error.
    $usuario = User::findOrFail($id);

    // comprobamos que el usuario sea administrador o intente acceder a sus propias etiquetas
    if (auth()->user()->esAdministrador() || auth()->user()->id == $usuario->id ){
      // Redirigimos a la vista de generación masiva de etiquetas con los datos del usuario.    
      return view('obras.etiquetas-por-lotes',compact('usuario'));
    }else{
      // No hay permiso
      abort(500, 'El usuario no tiene permisos suficientes para generar las etiquetas.');
    }
    
  }
  
  /**
  * Mostramos la vista con la etiqueta de la obra. La etiqueta se genera en la vista.
  */
  public function ver_etiqueta($id)
  {
    // Localizamos la obra o lanzamos error.
    $obra = Obra::findOrFail($id);
    
    // Marcamos que la obra tiene etiqueta en la DB y actualizamos el registro.
    // Luego, redirigimos a la vista de la etiqueta con los datos de la obra especificada por $id.    
    $obra->tieneEtiqueta = true;
    $obra->update();
    return view('obras.etiqueta',compact('obra'));
  }
  
  /**
  * Gestionamos las subidas de imágenes de las obras. Aquí almacenmos la imagen y devolvemos la ruta de almacenamiento.
  * La persistencia en DB se hace mediante el formulario de creación/edición de obra.
  */
  public function subir_foto(User $usuario)
  {
    // Validamos que el archivo recibido en request->fotoObra cumpla los requisitos necesarios.
    $this->validate(request(),[
      'fotoObra' => 'required|image|max:2048|dimensions:min_width=600,min_height=600',
      // archivo de tipo imagen | máx. 2048 kB | mínimo 600x600
      ]
    );
    // Almacenamos la imagen recibida y validada en almacenamiento. La almacenamos en público para poder servir la imagen públicamente a posteriori.
    $fotoObra = request()->file('fotoObra')->store('public/enlazart/'.$usuario->id);
    // Obtenemos la ruta a la imagen guardada y la retornamos.      
    $fotoURL = Storage::url($fotoObra);
    return $fotoURL;
  }
  
  /**
  * Gestionamos el borrado de imágenes de las obras.
  */
  public function borrar_foto(User $usuario)
  {
    // Recibimos la imagen que se desea borrar. Hay que actualizar la ruta que hemos registrado en DB por la
    //  ruta de almacenamiento.
    $fotoObra = str_replace('storage','public',request()->fotoURL);
    
    // La imagen puede estar asociada ya a una obra o no. Aquí lo comprobamos y si está asociada a una obra, actualizamos las rutas a imágenes de la obra en DB.
    if (request()->hayObra)
    {
      $obra = Obra::findOrFail(request()->id_obra);
      $obra->Foto = null;
      $obra->Thumb = null;
      $obra->save();
    }
    
    // Borramos la imagen y devolvemos el resultado.
    $resultado = Storage::delete($fotoObra);
    return strval($resultado);
  }
  
  /**
  * Publicar la ficha pública de la obra.
  */
  public function publicar_ficha($id)
  {
    // Localizamos la obra o lanzamos error
    $obra = Obra::findOrFail($id);
    
    // ¿Tiene el usuario autenticado permiso para publicar la ficha pública de la obra?
    // La lógica de autorización está en app/Policies/ObraPolicy.php
    $this->authorize('publicar_ficha', $obra);
    
    // Para publicar la obra, se requiere que tenga una imagen asociada
    if ($obra->Thumb != null) {
      // Tiene imagen -> publicamos la obra, actualizamos la DB y redireccionamos al listado de obras con feedback.
      $obra->esPublica = true;
      $obra->save();
      return redirect()
      ->route('obras.index')
      ->with(['tipo' =>'success','info' =>'Obra publicada correctamente']);
    }
    
    // No tiene imagen --> No publicamos y damos feedback al usuario.
    return redirect()
    ->route('obras.index')
    ->with(['tipo' => 'danger', 'info' => 'Atención: la obra necesita una imagen para poder publicarse' ]);
  }
  
  /**
  * Ocultar la ficha pública de la obra.
  */
  public function ocultar_ficha($id)
  {
    // Localizamos la obra o lanzamos error
    $obra = Obra::findOrFail($id);
    // ¿Tiene el usuario autenticado permiso para ocultar la ficha pública de la obra?
    // La lógica de autorización está en app/Policies/ObraPolicy.php
    $this->authorize('ocultar_ficha', $obra);
    // Ocultamos la ficha, actualizamos DB y redirigimos con feedback.
    $obra->esPublica = false;
    $obra->save();
    return redirect()
    ->route('obras.index')
    ->with(['tipo' =>'success','info' =>'Ficha de la obra ocultada correctamente']);
  }
}
