<?php

// Controlador que alberga la lógica de gestión de usuarios, excepto la lógica de autorización que está en app/http/controllers/auth/

namespace App\Http\Controllers;

use Illuminate\Http\Request;

use App\User;
use App\Rol;
use App\Obra;
use App\Http\Requests\ActualizarUsuarioRequest;

class UsuariosController extends Controller
{
  // El uso de este controlador requerirá de la superación del middleware auth, es decir,
  // que el usuario haya iniciado sesión en la aplicación.
  // Algunos métodos, además, sólo serán accesibles para usurios con el rol 'admin' (administradores). Esto se valida con el 
  // middleware de desarrollo propio ubicado en app/Http/Middleware/ComprobarRolUsuario.php
  function __construct(){
    $this->middleware('auth');
    $this->middleware('roles:admin',['except'=>['edit','update','show','destroy','ver_favoritos']]);
  }
  
  /**
  * Listado de usuarios. Sólo para administradores
  */
  public function index()
  {
    // Obtenemos todos los usuarios y los mostramos en usuarios.index
    $usuarios = User::all();
    return view('usuarios.index',compact('usuarios'));
  }
  
  /**
  * Mostramos el usuario especificado por $id.
  */
  public function show($id)
  {
    // Localizamos el usuario o lanzamos error.
    $usuario = User::findOrFail($id);
    // ¿Tiene el usuario autenticado permiso para ver el usuario?
    // La lógica de autorización está en app/Policies/UsuarioPolicy.php
    $this->authorize('show',$usuario);
    // Mostramos el usuario solicitado con los datos de sus obras.
    $obras = $usuario->obras;
    return view('usuarios.show',compact('usuario','obras'));
  }
  
  /**
  * Edición del usuario especificado por $id
  */
  public function edit($id)
  {
    // Localiamos el usuario o lanzamos error
    $usuario = User::findOrFail($id);
    // ¿Tiene el usuario autenticado permiso para ver ficha de edición del usuario especificado?
    // La lógica de autorización está en app/Policies/UsuarioPolicy.php
    $this->authorize($usuario);
    
    // Ajustamos los planes disponibles para el usuario especificado.
    if ($usuario->esAdministrador())
    {
      // Si es administrador, todos los planes están habilitados
      $roles = Rol::all()->pluck('nombre','id');
    }else{
      // Si no es administrador, todos los planes excepto "Administrador"
      $roles = Rol::all()->whereNotIn('id', [1,1])->pluck('nombre','id');
    }
    
    // Mostramos la vista del formulario de edición
    return view('usuarios.edit',compact('usuario','roles'));
  }
  
  /**
  * Actualizar usuario según $request
  * Para validar los datos recibidos, se ha implementado la lógica de validación en una clase Request,
  * ubicada en app/Http/Requests/ActualizarUsuarioRequest.php
  */
  public function update(ActualizarUsuarioRequest $request, User $usuario)
  {
    // ¿Tiene el usuario autenticado permiso para editar el usuario especificado?
    // La lógica de autorización está en app/Policies/UsuarioPolicy.php
    $this->authorize($usuario);
    
    // Si tiene autorización, actualizamos los datos validados del $request
    $usuario->update( $request->validated() );
    
    // Redirigimos al listado de usuarios con feedback.
    return redirect()
    ->route('usuarios.index')
    ->with(['tipo' =>'success','info' =>'Cuenta de usuario actualizada']);
  }
  
  /**
  * Borrar la cuenta del usuario $id
  */
  public function destroy($id)
  {
    // Localizamos el usuario o lanzamos error
    $usuario = User::findOrFail($id);
    // ¿Tiene el usuario autenticado permiso para borrar el usuario especificado?
    // La lógica de autorización está en app/Policies/UsuarioPolicy.php
    $this->authorize('destroy',$usuario);
    // Tiene autorización -> borramos el $usuario
    $usuario->delete();
    // Redirigimos al listado de usuarios con feedback.
    return redirect()
    ->route('usuarios.index')
    ->with(['tipo' =>'success','info' =>'Cuenta de usuario eliminada']);
  }
  
  /**
  * Ver obras favoritas del usuario.
  */
  public function ver_favoritos($id)
  {
    // Localizamos el usuario o lanzamos error
    $usuario = User::findOrFail($id);
    $favs = $usuario->obras_etiquetadas(1);
    $likes = $usuario->obras_etiquetadas(2);
    return view('usuarios.favoritos',compact('favs','likes'));
  }
  
}
