<?php

/* Definición del modelo Usuario ==> equivale a un Usuario */


namespace App;

use Illuminate\Notifications\Notifiable;
use App\Notifications\ResetPasswordNotification;
use Illuminate\Foundation\Auth\User as Authenticatable;

class User extends Authenticatable
{
  use Notifiable;
  
  // especificamos los campos que podemos rellenar mediante asignación masiva
  protected $fillable = [
    'name', 'email', 'rol_id', 'password',
  ];
  
  // especificamos los campos que irán ocultos en los requests http
  protected $hidden = [
    'password', 'remember_token',
  ];
  
  
  // Un usuario puede aplicar muchas AccionesSociales() (etiquetas a obras)
  public function acciones_sociales()
  {
    return $this->hasMany(AccionSocial::class);
  }
  
  // Obtener el número máximo de colecciones del usuario, de acuerdo a su plan de usuario (rol)
  public function colecciones_disponibles()
  {
    return $this->rol->max_colecciones;
  }
  
  // Obtener el número de colecciones publicadas del usuario
  public function colecciones_publicas()
  {
    return $this
    ->exposiciones
    ->where('esPublica',true)
    ->count();
  }
  
  // Comprobar si el usuario es o no administrador
  public function esAdministrador()
  {
    return $this->tieneRoles(['admin']);
  }
  
  // Un usaurio puede tener muchas colecciones
  public function exposiciones()
  {
    return $this->hasMany(Exposicion::class);
  }
  
  // Obtener el número máximo de días de publicación de la ficha de colección, de acuerdo a su plan de usuario (rol)
  public function dias_plan()
  {
    return $this->rol->max_dias_duracion;
  }
  
  // Comprobar si el usuario ha aplicado la etiqueta $etiqueta a la obra $obra
  // 0 --> No ha aplicado.
  // !0 --> Ha aplicado.
  public function ha_etiquetado(Obra $obra, int $etiqueta)
  {
    return $this
    ->acciones_sociales
    ->where('etiqueta_id', $etiqueta)
    ->where('obra_id', $obra->id)
    ->count();
  }
  
  // Un usuario puede tener muchas obras.
  public function obras()
  {
    return $this->hasMany(Obra::class);
  }
  
  // Obtener las obras etiquetadas por el usuario
  public function obras_etiquetadas(int $etiqueta)
  {
    return $this
    ->acciones_sociales
    ->where('etiqueta_id',$etiqueta);
  }

  // Obtener el número máximo de obras que puede publicar el usuario, de acuerdo a su plan de usuario (rol)
  public function obras_plan()
  {
    return $this->rol->max_obras;
  }
  
  // Obtener el número de obras publicadas (con acceso público) por el usuario
  public function obras_publicadas()
  {
    return $this
    ->obras
    ->where('esPublica',true)
    ->count();
  }
  
  // Un usuario tiene 1 rol (plan de usuario)
  public function rol()
  {
    return $this->belongsTo(Rol::class);
  }
  
  /**
  * Sobreescribimos esta función para poder personalizar el email enviado al usuario en el restablecimiento de contraseña
  */
  public function sendPasswordResetNotification($token)
  {
    $this->notify(new ResetPasswordNotification($token));
  }
  
  // Definimos un Mutator (mutador) de Laravel para que antes de escribir en DB el campo $password se ejecute
  public function setPasswordAttribute($password)
  {
    // Este método se ha documentado y explicado en la memoria del TFG
    
    // Comprobamos si $password viene ya cifrado (hash (Regex: cadena es '$2y$##$' y 50+ caracteres))
    if ( preg_match('/^\$2y\$[0-9]*\$.{50,}$/', $password) ) {
      // ya es hash -> no volvemos a encriptar
      $this->attributes['password'] = $password;
    }
    else {
      // Si no viene cifrado (es texto plano) -> encriptar
      $this->attributes['password'] = bcrypt($password);
    }
  }
  
  // Comprobar si el usuario tiene al menos 1 de los roles especificados en el array $roles
  public function tieneRoles(array $roles)
  {
    foreach ($roles as $rol)
    {
      if ($this->rol->clave === $rol){
        return true;
      }
    }
    return false;
  }
  
}
